package com.hys.app.service.erp.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hys.app.converter.erp.OrderConverter;
import com.hys.app.framework.database.WebPage;
import com.hys.app.framework.database.mybatisplus.base.BaseServiceImpl;
import com.hys.app.framework.exception.ServiceException;
import com.hys.app.framework.util.BeanUtils;
import com.hys.app.mapper.erp.OrderMapper;
import com.hys.app.mapper.erp.WarehouseOutMapper;
import com.hys.app.mapper.goods.GoodsMapper;
import com.hys.app.model.erp.dos.*;
import com.hys.app.model.erp.dto.OrderDTO;
import com.hys.app.model.erp.dto.OrderItemDTO;
import com.hys.app.model.erp.dto.OrderQueryParams;
import com.hys.app.model.erp.dto.WarehouseOutShipDTO;
import com.hys.app.model.erp.enums.*;
import com.hys.app.model.erp.vo.GoodsVO;
import com.hys.app.model.erp.vo.OrderVO;
import com.hys.app.model.erp.vo.ProductVO;
import com.hys.app.service.erp.*;
import com.hys.app.service.kuaidiniao.KuaiDiNiaoService;
import com.hys.app.service.kuaidiniao.entity.AddressDTO;
import com.hys.app.service.kuaidiniao.entity.PreOrderModel;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 订单业务层实现
 *
 * @author 张崧
 * 2024-01-24 15:58:31
 */
@Service
@AllArgsConstructor
public class OrderManagerImpl extends BaseServiceImpl<OrderMapper, OrderDO> implements OrderManager {

    private OrderConverter converter;
    private GoodsMapper goodsMapper;
    private OrderItemManager orderItemManager;
    private ProductStockManager productStockManager;
    private NoGenerateManager noGenerateManager;
    private WarehouseOutMapper warehouseOutMapper;
    private WarehouseOutItemManager warehouseOutItemManager;
    private ProductManager productManager;

    @Override
    public WebPage<OrderVO> list(OrderQueryParams queryParams) {
        WebPage<OrderDO> page = baseMapper.selectPage(queryParams);
        return converter.convertPage(page);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long add(OrderDTO orderDTO) {
        if (OrderTypeEnum.TO_C.equals(orderDTO.getType())) {
            if (ObjectUtil.isNull(orderDTO.getStartTime()) || ObjectUtil.isNull(orderDTO.getEndTime())) {
                throw new ServiceException("租用时间不能为空");
            }
        }
        OrderDO orderDO = BeanUtil.copyProperties(orderDTO, OrderDO.class);
        orderDO.setSn(noGenerateManager.generate(NoBusinessTypeEnum.Order, orderDO.getDeptId()));
        // 保存订单
        save(orderDO);
        // 保存订单项
        List<OrderItemDO> itemList = BeanUtils.toBean(orderDTO.getItemList(), OrderItemDO.class);
        //校验库存
        checkStock(orderDTO.getWarehouseId(), itemList);
        itemList.forEach(item -> item.setOrderId(orderDO.getId()));
        orderItemManager.saveBatch(itemList);
        return orderDO.getId();
    }

    private void checkStock(Long warehouseId, List<OrderItemDO> itemList) {
        //订单后面需要发货订单的数量不等你超过可用库存的数量
        List<Long> productId = itemList.stream().map(OrderItemDO::getProductId).collect(Collectors.toList());
        List<ProductStockDO> stockDOList = productStockManager.list(new LambdaQueryWrapper<ProductStockDO>()
                .eq(ProductStockDO::getWarehouseId, warehouseId)
                .in(ProductStockDO::getProductId, productId));
        for (ProductStockDO stockDO : stockDOList) {
            OrderItemDO dto = itemList.stream().filter(item -> item.getProductId().equals(stockDO.getProductId())).findFirst().get();
            if (dto.getNum() > stockDO.getUsableStock()) {
                throw new ServiceException(dto.getProductId() + "的可用库存数量不足");
            }
            stockDO.setUsableStock(stockDO.getUsableStock() - dto.getNum());
        }
        productStockManager.updateBatchById(stockDOList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void edit(OrderDTO orderDTO) {
        if (OrderTypeEnum.TO_C.equals(orderDTO.getType())) {
            if (ObjectUtil.isNull(orderDTO.getStartTime()) || ObjectUtil.isNull(orderDTO.getEndTime())) {
                throw new ServiceException("租用时间不能为空");
            }
        }
        OrderDO orderDO = BeanUtil.copyProperties(orderDTO, OrderDO.class);
        // 更新订单
        updateById(orderDO);
        // 更新订单项
        List<OrderItemDO> itemList = BeanUtils.toBean(orderDTO.getItemList(), OrderItemDO.class);
        //校验库存
        checkStock(orderDTO.getWarehouseId(), itemList);
        itemList.forEach(item -> item.setOrderId(orderDO.getId()));
        orderItemManager.remove(new LambdaQueryWrapper<OrderItemDO>().eq(OrderItemDO::getOrderId, orderDTO.getId()));
        orderItemManager.saveBatch(itemList);
    }

    @Override
    public OrderVO getDetail(Long id) {
        OrderDO orderDO = getById(id);
        OrderVO orderVO = BeanUtil.copyProperties(orderDO, OrderVO.class);
        List<OrderItemDO> list = orderItemManager.list(new LambdaQueryWrapper<OrderItemDO>().eq(OrderItemDO::getOrderId, id));
        List<Long> goodIdlist = list.stream().map(OrderItemDO::getProductId).collect(Collectors.toList());
        // 查询订单项
        List<OrderItemDTO> itemList = BeanUtils.toBean(list, OrderItemDTO.class);
        //所有的商品信息
        List<ProductDO> productDOList = productManager.list(new LambdaQueryWrapper<ProductDO>().in(ProductDO::getId, goodIdlist));
        List<ProductStockDO> stockDOList = productStockManager.list(new LambdaQueryWrapper<ProductStockDO>()
                .eq(ProductStockDO::getWarehouseId, orderDO.getWarehouseId()).in(ProductStockDO::getProductId, goodIdlist));
        for (OrderItemDTO dto : itemList) {
            ProductVO productVO = BeanUtil.copyProperties(productDOList.stream().filter(item -> item.getId().equals(dto.getProductId())).findFirst().get(), ProductVO.class);
            dto.setProductVO(productVO);
            GoodsVO goodsVO = BeanUtil.copyProperties(goodsMapper.selectById(productVO.getGoodsId()), GoodsVO.class);
            dto.setGoodsVO(goodsVO);
            dto.setUsableStock(stockDOList.stream().filter(item -> item.getProductId().equals(dto.getProductId())).findFirst().get().getUsableStock());
        }
        orderVO.setItemList(itemList);
        return orderVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long id) {
        OrderDO aDo = getById(id);
        if ("Y".equals(aDo.getHasSubmit())) {
            throw new ServiceException("该商品已确认");
        }
        removeById(id);
        //删除子项
        orderItemManager.remove(new LambdaQueryWrapper<OrderItemDO>().eq(OrderItemDO::getOrderId, id));
    }

    @Override
    public void warehouseOutShip(WarehouseOutShipDTO shipDTO) {
        lambdaUpdate()
                .set(OrderDO::getStatus, OrderStatusEnum.COMPLETE)
                .set(OrderDO::getLogisticsTrackingNumber, shipDTO.getTrackingNumber())
                .eq(OrderDO::getWarehouseOutId, shipDTO.getId())
                .update();
    }

    @Override
    public void submit(Long id) {
        OrderDO aDo = getById(id);
        if (!SaleOrderStatusEnum.TO_BE_COMMIT.equals(aDo.getStatus())) {
            throw new ServiceException("该商品已确认");
        }
        aDo.setStatus(SaleOrderStatusEnum.TO_BE_SEND);
        updateById(aDo);
    }

    @Override
    public IPage<OrderVO> hasSendOrder(OrderQueryParams queryParams) {
        Page<OrderDO> page = new Page<>(queryParams.getPageNo(), queryParams.getPageSize());
        LambdaQueryWrapper<OrderDO> wrapper = new LambdaQueryWrapper<OrderDO>()
                .eq(ObjectUtil.isNotNull(queryParams.getWarehouseId()), OrderDO::getWarehouseId, queryParams.getWarehouseId())
                .eq(StrUtil.isNotEmpty(queryParams.getSn()), OrderDO::getSn, queryParams.getSn());
        switch (queryParams.getSelectType()) {
            case "to_be_send":
                wrapper.in(OrderDO::getStatus, SaleOrderStatusEnum.TO_BE_SEND.getValue());
                break;
            case "sending":
                wrapper.notIn(OrderDO::getStatus, ListUtil.toList(SaleOrderStatusEnum.TO_BE_COMMIT.getValue(), SaleOrderStatusEnum.TO_BE_SEND.getValue()));
                break;
        }
        Page<OrderDO> doPage = page(page, wrapper);
        IPage<OrderVO> voPage = new Page<>(queryParams.getPageNo(), queryParams.getPageSize());
        voPage.setSize(doPage.getSize());
        voPage.setPages(doPage.getPages());
        voPage.setRecords(BeanUtils.toBean(doPage.getRecords(), OrderVO.class));
        voPage.setTotal(doPage.getTotal());
        return voPage;
    }


    @Override
    @Transactional
    public void orderSend(Long orderId, AddressDTO addressDTO) {
        OrderDO aDo = getById(orderId);
        aDo.setSendAddress(addressDTO.getSendAddress());
        aDo.setReceiverAddress(addressDTO.getReceiveAddress());
        //根据订单查找到这个订单的所有货物扣减库存
        List<OrderItemDO> orderItemDOS = orderItemManager.list(new LambdaQueryWrapper<OrderItemDO>().eq(OrderItemDO::getOrderId, orderId));
        List<Long> productId = orderItemDOS.stream().map(OrderItemDO::getProductId).collect(Collectors.toList());
        //查找这些商品在指定仓库的库存，减去对应数量
        List<ProductStockDO> stockDOList = productStockManager.list(new LambdaQueryWrapper<ProductStockDO>()
                .eq(ProductStockDO::getWarehouseId, aDo.getWarehouseId()).in(ProductStockDO::getProductId, productId));
        for (ProductStockDO stockDO : stockDOList) {
            Integer num = orderItemDOS.stream().filter(item -> item.getProductId().equals(stockDO.getProductId())).findFirst().get().getNum();

            if (num < stockDO.getUsableStock()) {
                if (OrderTypeEnum.TO_B.equals(aDo.getType())) {
                    //扣减可用库存
//                    stockDO.setUsableStock(stockDO.getUsableStock() - num);
                    //扣减实际库存
                    stockDO.setActualStock(stockDO.getActualStock() - num);
                } else if (OrderTypeEnum.TO_C.equals(aDo.getType())) {
                    //扣减可用库存
                    stockDO.setActualStock(stockDO.getUsableStock() - num);
                    //增加在途库存
                    stockDO.setTransitStock(stockDO.getTransitStock() + num);
                }
            } else {
                throw new ServiceException(stockDO.getProductId() + "的库存不够");
            }
        }
        productStockManager.updateBatchById(stockDOList);
        //生成出库单
        WarehouseOutDO outDO = new WarehouseOutDO();
        outDO.setType(WarehouseOutTypeEnum.BORROW_OUT);
        outDO.setOrderIdList(orderId);
        outDO.setWarehouseId(aDo.getWarehouseId());
        outDO.setNotes("发货单生成的出货单");
        outDO.setSn(noGenerateManager.generate(NoBusinessTypeEnum.WarehouseOut, 66L));
        warehouseOutMapper.insert(outDO);
        //保存出库单明细
        List<WarehouseOutItemDO> beanList = BeanUtils.toBean(orderItemDOS, WarehouseOutItemDO.class);
        beanList.forEach(item -> item.setWarehouseOutId(outDO.getId()));
        warehouseOutItemManager.saveBatch(beanList);
        //更改订单状态56789=m+

        if (SaleOrderStatusEnum.TO_BE_SEND.equals(aDo.getStatus())) {
            aDo.setStatus(SaleOrderStatusEnum.SENDING);
            aDo.setWarehouseOutId(outDO.getId());
            updateById(aDo);
        } else {
            throw new ServiceException("订单状态错误,待发货的订单才能发货");
        }
    }
}

